@@ -44,17 +44,17 @@
 
 /* PDF and CDF of 7-param diffusion */
   /* PDF */
-  void PDF7(int, double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, int);
+  void PDF7(int, double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, int, int);
 
   /* CDF */
-  void CDF7(int, double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, int);
+  void CDF7(int, double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, int, int);
 /* ------------------------------------------------ */
 
 /* gradient of PDF and CDF */
   /* derivative of PDF with respect to all params */
-  void dxPDF7(double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, double *, double *, double *, double *, double *, int);
+  void dxPDF7(double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, double *, double *, double *, double *, double *, int, int);
   /* derivative of CDF with respect to all params */
-  void dxCDF7(double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, double *, double *, double *, double *, double *, int);
+  void dxCDF7(double *, int *, double *, double *, double *, double *, double *, double *, double *, double, int, int, int, double *, double *, double *, double *, double *, double *, double *, double *, int, int);
 /* ------------------------------------------------ */
 
 
