@@ -5,6 +5,9 @@
 #include <Rinternals.h>
 #include <cmath>
 #include "tools.h"
+#include <mutex>
+
+std::mutex mtx_samp;
 
 
 double logsum(double xa, double xb) {
@@ -149,7 +152,7 @@ double coth(double x) {
 
 double lower_bound_time(double a, double vn, double wn) {
 	double temp, amw = a * (1 - wn);
-	if (std::fabs(vn) < 1e-5) {
+	if (fabs(vn) < 1e-5) {
 		temp = (pow(a, 2) - pow(amw, 2)) / 3.0;
 
 	}
@@ -168,6 +171,7 @@ double exp_mean(int pm, double a, double v, double w) {
 
 double oneuni() {
 	double u;
+	std::lock_guard<std::mutex> guard(mtx_samp);
 	do {
 		GetRNGstate();
 		u = unif_rand();
@@ -179,6 +183,7 @@ double oneuni() {
 
 double oneuniL() {
 	double u;
+	std::lock_guard<std::mutex> guard(mtx_samp);
 	do {
 		GetRNGstate();
 		u = unif_rand();
@@ -228,6 +233,7 @@ Copyright (C) 2002 Przemyslaw Sliwa and Jason H. Stover. GPL 3
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Gerard Jungman. GPL 3
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Gerard Jungman, Brian Gough. GPL 3
 
+
 */
 
 double rat_eval(const double a[], const size_t na,
@@ -351,18 +357,18 @@ static inline int cheb_eval_e(const cheb_series * cs,
   for(j = cs->order; j>=1; j--) {
     double temp = d;
     d = y2*d - dd + cs->c[j];
-    e += std::fabs(y2*temp) + std::fabs(dd) + std::fabs(cs->c[j]);
+    e += fabs(y2*temp) + fabs(dd) + fabs(cs->c[j]);
     dd = temp;
   }
 
   {
     double temp = d;
     d = y*d - dd + 0.5 * cs->c[0];
-    e += std::fabs(y*temp) + std::fabs(dd) + 0.5 * std::fabs(cs->c[0]);
+    e += fabs(y*temp) + fabs(dd) + 0.5 * fabs(cs->c[0]);
   }
 
   result->val = d;
-  result->err = GSL_DBL_EPSILON * e + std::fabs(cs->c[cs->order]);
+  result->err = GSL_DBL_EPSILON * e + fabs(cs->c[cs->order]);
 
   return GSL_SUCCESS;
 }
@@ -601,7 +607,7 @@ inline static double erfc8(double x)
 }
 
 int gsl_sf_erfc_e(double x, gsl_sf_result * result) {
-  const double ax = std::fabs(x);
+  const double ax = fabs(x);
   double e_val, e_err;
 
   /* CHECK_POINTER(result) */
@@ -619,7 +625,7 @@ int gsl_sf_erfc_e(double x, gsl_sf_result * result) {
     gsl_sf_result c;
     cheb_eval_e(&erfc_x15_cs, t, &c);
     e_val = ex2 * c.val;
-    e_err = ex2 * (c.err + 2.0*std::fabs(x)*GSL_DBL_EPSILON);
+    e_err = ex2 * (c.err + 2.0*fabs(x)*GSL_DBL_EPSILON);
   }
   else if(ax < 10.0) {
     double exterm = exp(-x*x) / ax;
@@ -627,22 +633,22 @@ int gsl_sf_erfc_e(double x, gsl_sf_result * result) {
     gsl_sf_result c;
     cheb_eval_e(&erfc_x510_cs, t, &c);
     e_val = exterm * c.val;
-    e_err = exterm * (c.err + 2.0*std::fabs(x)*GSL_DBL_EPSILON + GSL_DBL_EPSILON);
+    e_err = exterm * (c.err + 2.0*fabs(x)*GSL_DBL_EPSILON + GSL_DBL_EPSILON);
   }
   else {
     e_val = erfc8(ax);
-    e_err = (x*x + 1.0) * GSL_DBL_EPSILON * std::fabs(e_val);
+    e_err = (x*x + 1.0) * GSL_DBL_EPSILON * fabs(e_val);
   }
 
   if(x < 0.0) {
     result->val  = 2.0 - e_val;
     result->err  = e_err;
-    result->err += 2.0 * GSL_DBL_EPSILON * std::fabs(result->val);
+    result->err += 2.0 * GSL_DBL_EPSILON * fabs(result->val);
   }
   else {
     result->val  = e_val;
     result->err  = e_err;
-    result->err += 2.0 * GSL_DBL_EPSILON * std::fabs(result->val);
+    result->err += 2.0 * GSL_DBL_EPSILON * fabs(result->val);
   }
 
   return GSL_SUCCESS;
@@ -655,3 +661,12 @@ double gsl_sf_erfc(double x)
 
 
 /* ----------------------------------- */
+
+
+void check_interruption(void *ptr) {
+  R_CheckUserInterrupt();
+}
+
+int is_interruption() {
+  return !(R_ToplevelExec(check_interruption, nullptr));
+}
