@@ -5,9 +5,13 @@
 #ifndef TOOLS_H
 #define TOOLS_H
 
-#include "cmath"
+#include <cmath>
 #include <vector>
 #include <R.h>
+// #include <mutex>
+//
+// extern std::mutex mtx_R_CUI; // mutex for R_CheckUserInterrupt
+
 
 /*------------------------------------------------*/
 
@@ -94,9 +98,6 @@ struct ars_archiv {
   /* The following mathematical constants
   are copied from GSL version 2.6 */
 
-  /* The following mathematical constants
-  are copied from GSL version 2.6 */
-
   // Copyright (C) 1996, 1997, 1998, 1999, 2000, 2004, 2007 Gerard Jungman, Brian Gough. GPL 3
 
 #ifndef GSL_DBL_EPSILON
@@ -237,6 +238,8 @@ double coth(double);
 double lower_bound_time(double, double, double);
 double exp_mean(int, double, double, double);
 
+int is_interruption();
+
 
 
 #endif
